//
//  DCUcontroller.h
//  DCU
//
//  Created by Christoph Birk on 2013-02-27
//  Copyright 2013 Carnegie Observatories. All rights reserved.
//

/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

#import "FFwidgets.h"
#import "FFmodel.h"
#import "telio.h"
#import "plcio.h"
#import "Logger.h"

/* ---------------------------------------------------------------- */

#define N_LAMPS         8

#define DCU_MCAL        0
#define DCU_FFS         1

#define KEY_ENGMODE     @"eng_mode"
#define KEY_FFS_DSBLD   @"ffs_disabled"
#define KEY_MCAL_DSBLD  @"mcal_disabled"

const char* ErrorText(int err);

/* ---------------------------------------------------------------- */

@interface DCUcontroller : NSObject {
  IBOutlet NSWindow *window;           // main window
           NSString *s_title;
           char     p_title[32];
           NSString *telname;
           char     telhost[128];
           TEL_IO   *tcs;
           PLC_IO   *plc;
           u_long   counter;
           u_int    cookie;
  volatile BOOL     configured;
           BOOL     use_interlocks,eng_mode;
           time_t   next_dog,next_air;
           int      interval_dog;
           float    air_pressure,min_pressure;
           Logger   *logger;
           FFcoil   *ff_enable,*mc_enable;
           int      selector_addr;
           int      cookie_addr;
           int      error_state;
           int      error_addr;
           BOOL     ff_disabled,mc_disabled; // NEW v0.904
  // GUI - lamps
           FFlamp   *lamps[N_LAMPS];
  IBOutlet NSButton *but_lamp1,*but_lamp2,*but_lamp3;
  IBOutlet NSButton *but_lamp4,*but_lamp5,*but_lamp6;
  IBOutlet NSButton *but_lamp7,*but_lamp8;
           NSArray  *but_lamps;
           NSImage  *icon_on,*icon_off,*icon_err;
  // GUI - variable lamps
           FFvarLamp     *q_lamp;
  IBOutlet NSPopUpButton *popup_quartz;
  IBOutlet NSTextField   *edit_quartz,*label_quartz;
  // GUI - FFS
           FFscreen      *ff_screen;
  IBOutlet NSPopUpButton *drop_ffs,*popup_ffbrake; 
  IBOutlet NSButton      *check_ffins,*check_ffret,*check_ffbrake;
  IBOutlet FFprogress    *prog_screen;
  
  // GUI - Mcal
           FFscreen      *mc_screen;
  IBOutlet NSPopUpButton *drop_mcal,*popup_mcbrake; 
  IBOutlet NSButton      *check_mcins,*check_mcret,*check_mcbrake;
  IBOutlet FFprogress    *prog_mcal;
  // GUI - Other
  IBOutlet NSTextField   *edit_air;
           FFsensor      *air_sensor;
}

@property (readonly,nonatomic) NSPopUpButton *drop_ffs,*drop_mcal;
@property (readonly)           int error_state;

/* ---------------------------------------------------------------- */

- (BOOL)configure;
- (void)run_conf:(id)param;
- (void)fireTimer:(NSTimer*)theTimer;
-  (int)getFullStatus;
- (void)shutdown;
- (BOOL)running;
- (BOOL)deployed;

- (void)action_lamp:(id)sender;
- (void)action_ffs:(id)sender;
- (void)action_mcal:(id)sender;

- (void)updateState:(BOOL)toTCS;
- (void)updateLamp:(int)lamp;
- (void)updateVar;
- (void)updateTCS;
- (void)updateFFS;
- (void)updateMcal;

- (void)run_dog;

- (BOOL)checkAir:(BOOL)interactive;
-  (int)checkPLC:(BOOL)interactive;

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
