//
//  acroio.m
//  DCU
//
//  Created by Christoph Birk on 3/20/13.
//  Copyright 2013 Carnegie Observatories. All rights reserved.
//

/* ---------------------------------------------------------------- */

#define DEBUG      1

/* ---------------------------------------------------------------- */

#import "acroio.h"

/* ---------------------------------------------------------------- */

@implementation Acromag_IO

/* ---------------------------------------------------------------- */

- (int)analogOut:(int)channel Value:(int)value
{
  int err,n;

#if (DEBUG > 0)
  fprintf(stderr,"%s(%d,%d)\n",PREFUN,channel,value);
#endif

  err = [self open:lock_timeout]; if (err) return err;

  n = [self send:0x06 Address:(0x12+channel) Value:value]; 
  if (online) {
    if (n < 1) err = E_acro_ioerror;
  }
  [self close];

  return err;
}

/* ---------------------------------------------------------------- */

- (int)analogIn:(int)channel Value:(int*)value
{
  int  err,n;
  char buf[256];
   
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d,%p)\n",PREFUN,channel,value);
#endif

  err = [self open:lock_timeout]; if (err) return err;

  n = [self send:0x04 Address:(0x06+channel) Value:0x0001 Response:buf];
  if (online) {
    if (n == 11) *value = 256*(u_char)buf[9] + (u_char)buf[10];
    else         err = E_acro_ioerror;
  }
  [self close];  

  return err;
}

/* ---------------------------------------------------------------- */

- (int)setCoil:(int)channel Status:(int)status
{
  int err,n;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d,%d)\n",PREFUN,channel,status);
#endif

  err = [self open:lock_timeout]; if (err) return err; // todo remvoe

  n = [self send:0x05 Address:(0x00+channel) Value:((status) ? 0xff00 : 0x0000)];
  if (online) {
    if (n < 1) err = E_acro_ioerror;
  }
  [self close];

  return err;
}

/* ---------------------------------------------------------------- */

- (int)getCoil:(int)channel Status:(int*)status
{
  int  err,n;
  char buf[256];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%d,%p)\n",PREFUN,channel,status);
#endif

  err = [self open:lock_timeout]; if (err) return err; // todo remove

  n = [self send:0x01 Address:(0x00+channel) Value:0x0001 Response:buf];
  if (online) {
    if (n == 10) *status = (int)buf[9];
    else         err = E_acro_ioerror;
  }
  [self close];

  return err;
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
