//
// AppDelegate.m
// DCU
//
// Created by Christoph Birk on 2013-02-26
// Copyright 2013 Carnegie Observatories. All rights reserved.
//

/* ---------------------------------------------------------------- */

#define DEBUG           1

#define FRAME_MAIN       @"mainWindow"

/* ---------------------------------------------------------------- */

#import "AppDelegate.h"
#import "Notice.h"

/* ---------------------------------------------------------------- */

@implementation AppDelegate

// @synthesize window;
@synthesize dcuController;

/* ---------------------------------------------------------------- */

- (void)applicationDidFinishLaunching:(NSNotification *)note
{
  NSString  *telname,*dcuname;
  id        status;
  int       value=0;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif
  assert(dcuController);
  
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

  assert(popup_telname);
  telname = [defaults stringForKey:KEY_TELNAME]; // printf("telname=%p\n",telname);
  if (telname) [popup_telname selectItemWithTitle:telname];
  else         [popup_telname selectItemWithTitle:@"Clay"];
  assert(popup_telstat);
  status = [defaults objectForKey:KEY_TELSTAT];
  value = (status) ? [status intValue] : 1;
  [popup_telstat selectItemAtIndex:value];
  if ([telname compare:@"Baade"] == NSOrderedSame) { // v0.904
    [check_mcal setState:NSOffState];
    [check_mcal setEnabled:NO];        
  }
  
  assert(combo_dcuname);
#ifdef SIM_ONLY
#ifndef NDEBUG
  [combo_dcuname addItemWithObjectValue:@"andromeda"];
#endif
  [combo_dcuname addItemWithObjectValue:@"localhost"];
#else
#ifndef NDEBUG
  [combo_dcuname addItemWithObjectValue:@"192.91.178.187"];
#endif
#endif
  dcuname  = [defaults stringForKey:KEY_DCUNAME]; // printf("dcuname=%p\n",dcuname);
  if (dcuname) [combo_dcuname setStringValue:dcuname];
  assert(popup_dcustat);
  status = [defaults objectForKey:KEY_DCUSTAT];
  value = (status) ? [status intValue] : 1;
  [popup_dcustat selectItemAtIndex:value];

  assert(check_engmode);               // v0.809
#ifdef NDEBUG
  status = [defaults objectForKey:KEY_ENGMODE];
  value = (status) ? [status intValue] : 1;
  [check_engmode setState:((value) ? NSOnState : NSOffState)];
#endif

  assert(check_ffs);                   // v0.904
  status = [defaults objectForKey:KEY_FFS_DSBLD];
  value = (status) ? [status intValue] : 0;
  [check_ffs setState:((value) ? NSOffState : NSOnState)];
  
  assert(check_mcal);                  // v0.904
  status = [defaults objectForKey:KEY_MCAL_DSBLD];
  value = (status) ? [status intValue] : 0;
  [check_mcal setState:((value) ? NSOffState : NSOnState)];

  assert(dcuWindow);
  [dcuWindow setFrameUsingName:FRAME_MAIN];
#ifdef NDEBUG
  [dcuWindow setStyleMask:([dcuWindow styleMask] & ~NSClosableWindowMask)];
#endif

  if (telname && dcuname) {            // got setup from preferences
#ifdef NDEBUG
    [self action_ok:self];
#endif
  } else {                             // interactive setup
    [self action_telname:popup_telname];
  }
}

/* ---------------------------------------------------------------- */

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

  return YES;
}

/* ---------------------------------------------------------------- */

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

#ifdef NDEBUG
  if ([dcuController running]) {     // v0.808
    [Notice openNotice:"DCU moving\rcancelled 'Quit'" Title:P_TITLE Time:30];
    return NSTerminateCancel;
  }
#endif
  
  return NSTerminateNow;
}

/* ---------------------------------------------------------------- */

- (void)applicationWillTerminate:(NSNotification *)note
{  
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif 
  
  [dcuController shutdown];
  [dcuWindow saveFrameUsingName:FRAME_MAIN];
}

/* ---------------------------------------------------------------- */

- (void)action_prefs:(id)sender
{
  if ([dcuController running]) return;
  
  assert([NSThread isMainThread]); 
  [window makeKeyAndOrderFront:self];
}

/* ---------------------------------------------------------------- */

- (void)action_telname:(id)sender;     // fix 'IB' defaults v1.001
{
  assert(sender == popup_telname);
  
  if ([[sender titleOfSelectedItem] compare:@"Baade"] == NSOrderedSame) {
    [combo_dcuname setStringValue:@"baadedcu"]; // 200.28.147.39
    [check_mcal setState:NSOffState];
    [check_mcal setEnabled:NO];        // v0.904
  } else 
  if ([[sender titleOfSelectedItem] compare:@"Clay"] == NSOrderedSame) {
    [combo_dcuname setStringValue:@"claydcu"];  // 200.28.147.40
    [check_mcal setEnabled:YES];
    [check_mcal setState:NSOnState];
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (void)action_ok:(id)sender
{
  BOOL     r;
  NSNumber *status;
  NSString *name;
  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
  
  name = [popup_telname titleOfSelectedItem];
  [defaults setObject:name forKey:KEY_TELNAME];
  status = [NSNumber numberWithInt:[popup_telstat indexOfSelectedItem]];
  [defaults setObject:status forKey:KEY_TELSTAT];

  name = [combo_dcuname stringValue];
  [defaults setObject:name forKey:KEY_DCUNAME];
  status = [NSNumber numberWithInt:[popup_dcustat indexOfSelectedItem]];
  [defaults setObject:status forKey:KEY_DCUSTAT];

  [defaults setBool:([check_engmode state] ? YES : NO) forKey:KEY_ENGMODE];
  [defaults setBool:([check_ffs state] ? NO : YES) forKey:KEY_FFS_DSBLD];
  [defaults setBool:([check_mcal state] ? NO : YES) forKey:KEY_MCAL_DSBLD];
  
  [defaults synchronize];
  
  assert(dcuController);
  r = [dcuController configure];
  if (r) {
    assert([NSThread isMainThread]); 
    [window orderOut:self];
    [dcuWindow makeKeyAndOrderFront:self];
  }
}

/* ---------------------------------------------------------------- */
/* Menu - Help */

- (IBAction)action_help:(id)sender     // v0.807
{
  NSURL *url=nil;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  url = [NSURL URLWithString:@"http://users.obs.carnegiescience.edu/birk/DCU"];
  if (url) [[NSWorkspace sharedWorkspace] openURL:url];
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
