//
// yutils.h
//
// Created by Christoph Birk on 2011-04-19
// Copyright 2011 Carnegie Observatories. All rights reserved.
//

/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

/* ---------------------------------------------------------------- */

#define SHARED_BASE  "/Users/Shared/Library"
#define SHARED_LOGS  "Logs"
#define SHARED_APPS  "Application Support"
#define SHARED_PREF  "Preferences"

/* ---------------------------------------------------------------- */
/* Utility functions */
/* ---------------------------------------------------------------- */

BOOL        YbundleInfo       (char**,char**,char**,char**,char**,char**);
const char* YbundleResource   (const char*,const char*);
NSArray*    YdirectoryURL     (NSUInteger);
char*       YappSupportCreate (char**);

char*       Ystr2chr (NSString*,char*,int);
const char* Ystr2tmp (NSString*);
NSString*   Ychr2str (const char*);

NSArray*  YarrayFromString (const char*,const char*);
NSArray*  YarrayFromStringQ(const char*);
NSArray*  YarrayFromFloats (float*,int);
NSArray*  YarrayFromInts   (int*,int);
int       YarrayToValues   (NSArray*,int*,int);

void      YeditText  (NSTextField*,const char*);
char*     YeditGet   (NSTextField*,char*,int);
void      YeditColor (NSTextField*,float,float,float,float);
NSTextField* YeditCreate(NSView*,int,int,int);
NSTextField* YlabelCreate(NSView*,int,int,int,const char*);

NSArray* YpopupFromXML(NSPopUpButton*,NSString*);
void YpopupFromArray(NSPopUpButton*,NSArray*);
void YpopupSelectItem(NSPopUpButton*,int);
void YpopupSetTitle  (NSPopUpButton*,const char*);
void YpopupEnable    (NSPopUpButton*,BOOL);
void YcontrolEnable  (NSControl*,BOOL);
void YviewNeedsDisplay(NSView* view);

void YwindowTitle    (NSWindow*,const char*);
void YwindowFrameSet (NSWindow*,const char*);
void YwindowFrameSave(NSWindow*,const char*);

void YmenuEnableTitles(const char*,BOOL);
NSMenuItem* YmenuItem (const char*);

NSImage* YimageFile  (NSImageView*,const char*); 
NSImage* YimageData  (NSImageView*,const u_int*,int,int);

// int YgetTableColumn(NSTableView*,NSTableColumn*);

BOOL      YprefSetupShared (void);
void      YprefPutInt      (const char*,int,BOOL);
int       YprefGetInt      (const char*,int,BOOL);
void      YprefPutDouble   (const char*,double,BOOL);
double    YprefGetDouble   (const char*,double,BOOL);
void      YprefPutChar     (const char*,const char*,BOOL);
char*     YprefGetChar     (const char*,const char*,char*,int,BOOL);
void      YprefPutString   (NSString*,NSString*,BOOL);
NSString* YprefGetString   (NSString*,NSString*,BOOL);
void      YprefPutArray    (const char*,NSArray*,BOOL);
NSArray*  YprefGetArray    (const char*,BOOL);

NSInteger YalertPanel (const char*,const char*,int,const char*,const char*,const char*);
BOOL      YalertQuestion (const char*,const char*,int,const char*,const char*);
// void YsheetQuestion(NSWindow*,const char*,const char*,id,SEL,void*);
void YsheetNotify(NSWindow*,const char*,const char*,id,SEL,void*);
// void YpanelNotify(const char*,const char*,int);
// void YpanelQuestion(const char*,const char*);
// void YerrorDialog   (const char*,const char*);
void      YerrorSheet (NSWindow*,const char*,const char*);

typedef NSRecursiveLock* Ymutex;
Ymutex  YmutexCreate(const char*);
BOOL    YmutexLock(Ymutex,double);
void    YmutexRelease(Ymutex);

typedef NSDistributedLock* Yglobal;
Yglobal YglobalCreate(const char*);
BOOL    YglobalLock(Yglobal,int,int);
void    YglobalRelease(Yglobal);

typedef struct { NSLock *lock; int count; } Ysemaphore; 
Ysemaphore* YsemaphoreCreate(const char*);
int YsemaphoreInc(Ysemaphore*);
int YsemaphoreDec(Ysemaphore*);
int YsemaphoreAdd(Ysemaphore*,int);
int YsemaphoreGet(Ysemaphore*);
int YsemaphoreIncLimit(Ysemaphore*,int);

BOOL YfileExists      (const char*);
BOOL YfileReadable    (const char*);
BOOL YpathReadable    (const char*);
BOOL YfileWritable    (const char*);
BOOL YpathWritable    (const char*);
BOOL YpathCreate      (const char*,BOOL);
BOOL YfileRemove      (const char*);
NSInteger YfileTrash  (const char*);
BOOL YfilePermissions (const char* file,short u,short g,short o);
BOOL YfileCopy        (const char* src, const char* trg);

void    Ysleep    (int);
NSTask* Ytask     (const char*,const char*);
// void Ysay      (const char*);
// void Ysound    (const char*);
void Ywww      (const char*);

void CtextSetColorText(const char* text,CTFontRef font,CGContextRef c,
                       float x,float y,float r,float g,float b,float a);

CTFontRef CtextGetFont(const char* family,float size);
CTLineRef CtextGetLine(const char* text,CTFontRef font);
void      CtextSetText(const char*,CTFontRef,CGContextRef,float,float);
void      CtextCenterText(const char*,CTFontRef,CGContextRef,float,float);
void      CtextRightText(const char*,CTFontRef,CGContextRef,float,float);

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
