/* ---------------------------------------------------------------- */
//
//  FFmodel.h
//  DCU
//
//  Created by Christoph Birk on 2013-03-15
//  Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>
#import "plcio.h"

/* ---------------------------------------------------------------- */

enum FF_error_enum { E_FF_error=E_plcio_last+1,
                     E_FF_coil,
                     E_FF_lamp,
                     E_FF_varLamp,
                     E_FF_brake,
                     E_FF_screen,
                     E_FF_last
};

/* --- */

@interface FFbase : NSObject {
  NSString  *name;
  PLC_IO    *connection;
  int       status;
}

@property (readonly,nonatomic,assign) NSString *name;
@property (readonly,nonatomic)        int      status;

+ (NSString*)errorString:(int)err;

- (id)initWithPLC:(PLC_IO*)con Name:(NSString*)nam;

@end

#define KEY_STATUS @"status"

/* ---------------------------------------------------------------- */

@interface FFcoil : FFbase {
  int  addr_coil,addr_out;
}

- (id)initWithPLC:(PLC_IO*)con XML:(NSDictionary*)dict Prefix:(NSString*)pref;
- (int)get;
- (int)set:(int)state;

@end

/* ---------------------------------------------------------------- */

@interface FFinput : FFbase {          // digtal input
  int addr_input;
}

- (id)initWithPLC:(PLC_IO*)con XML:(NSDictionary*)dict Prefix:(NSString*)pref;
- (int)get;
- (int)get:(int)value;

@end

/* ---------------------------------------------------------------- */

@interface FFsensor : FFinput {        // analog input
}

@end

/* ---------------------------------------------------------------- */

@interface FFdevice : FFbase {
  int  request;
  BOOL moving;
}

@property (readonly,nonatomic) int  request;
@property (readonly)           BOOL moving;

- (int)get;
- (void)run:(NSNumber*)target;

@end

#define KEY_MOVING @"moving"

/* ---------------------------------------------------------------- */

@interface FFlamp : FFdevice {
  FFcoil *coil;
}

- (id)initWithPLC:(PLC_IO*)con XML:(NSDictionary*)dict Prefix:(NSString*)pref;

@end

/* ---------------------------------------------------------------- */

@interface FFvarLamp : FFlamp {        // analog output
  int addr_analog;
  int level;                           // dimmer
}

@property (readonly,nonatomic) int level;

@end

#define LAMP_OFF  0
#define LAMP_ON   1

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */

@interface FFbrake : FFdevice {
  FFcoil  *coil;
  FFinput *pswitch;
}

@property (readonly,nonatomic) FFinput *pswitch;

@end

#define BRAKE_OFF  1
#define BRAKE_ON   0

/* ---------------------------------------------------------------- */

@interface FFscreen : FFdevice {       // FFS & MCal 
  FFcoil   *ins_coil,*ret_coil;
  FFinput  *ins_limit,*ret_limit;
  FFbrake  *brake;
}

- (id)initWithPLC:(PLC_IO*)con XML:(NSDictionary*)dict Prefix:(NSString*)pref;

@property (readonly,nonatomic) FFinput *ins_limit,*ret_limit;
@property (readonly,nonatomic) FFbrake *brake;

@end

#define SCREEN_RET  0x01
#define SCREEN_INS  0x02

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
