/* ---------------------------------------------------------------- */
//
// MTwister.h
//
// Created by Christoph Birk on 2013-04-12
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

/* ---------------------------------------------------------------- */

void   addRandomNumbers(NSMutableArray* array,int num);
u_int* hashBytes(u_char* bytes,int size);
void   addHashedUniqueString(NSMutableArray* array);

/* ---------------------------------------------------------------- */

@interface MTwister : NSObject {
  unsigned int *mt_state;
  volatile int  mt_index;
  int           mt_n;
  NSLock *instanceLock;
}

+ (MTwister*)defaultTwister;
+ (u_int)hash32FromFile:(const char*)file;
+ (u_int)hash32FromChars:(const char*)string;
+ (int)intValue:(int)x;
+ (double)doubleValue:(double)x;
+ (double)gaussMean:(double)mean Sigma:(double)sigma;
+ (u_int)secureUint;

- (id)init;
- (id)initWithLock:(BOOL)useLock;
- (id)initWithLock:(BOOL)useLock Urand:(int)n;
- (id)initWithFile:(const char*)file;
- (id)initWithBytes:(const char*)bytes length:(u_int)len;

- (void)setupWithBytes:(const char*)bytes length:(u_int)len;
- (void)setupState:(NSArray*)key;

- (int)intValue:(int)x;
- (u_int)uintValue;
- (double)doubleValue:(double)x;
- (double)quadValue:(double)x;
- (double)gaussMean:(double)mean Sigma:(double)sigma;
- (double)gaussQuad:(double)mean Sigma:(double)sigma;
- (u_int)secureUint;
- (u_int)mt_19937;

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
