/* ---------------------------------------------------------------- */
//
// modio.m
//
// Created by Christoph Birk on 2013-03-20
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import  "modio.h"

#include <sys/socket.h>

/* ---------------------------------------------------------------- */

@implementation Modbus_IO

/* ---------------------------------------------------------------- */
#pragma mark "Class"
/* ---------------------------------------------------------------- */

+ (NSString*)errorString:(int)err      // b1.0201
{
  NSString *string=nil;
  
  switch (err) {
    case E_modbus_error: string = @"ModbusIO: error"; break;
    default: string = [super errorString:err]; break;
  }
  return string;
}

/* ---------------------------------------------------------------- */
#pragma mark "Instance"
/* ---------------------------------------------------------------- */

- (int)send:(int)fc Address:(int)addr Value:(int)val
{
  return [self send:fc Address:addr Value:val Response:NULL];
}

/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (int)send:(int)fc Address:(int)addr Value:(int)val Response:(char*)res
{
  int     err;
  ssize_t n=0;
  char    message[32],buf[128];
  
  err = [self open:lock_timeout]; if (err) return -err;

  if (![self mylock:lock_timeout]) {
    [self close];
    return -E_tcpip_lock;
  }
  
  sprintf(message,"%c%c%c%c%c%c%c%c%c%c%c%c",
          0x00,0x00,0x00,0x00,0x00,0x06,  // fixed length = 6 bytes
          0x00,                        // unit ID
          fc,                          // function code
          addr/0x100,addr%0x100,       // address
          val/0x100,val%0x100);        // data

  if (online) {
    assert(sock > 0);
    if (send(sock,message,12,0) < 0) n = -E_tcpip_send;
    if (!n) {                          // no error
      assert(sock > 0);
      n = recv(sock,buf,sizeof(buf)-2,0);
      if (res && (n > 0)) { int i;
        for (i=0; i<n; i++) res[i] = buf[i];
        res[i] = '\0';
      }
    } else {
      n = -E_tcpip_recv;
    }
  } else {
    assert(sock == 0);
    [NSThread sleepForTimeInterval:0.050]; 
  }
  [lock unlock];
  [self close];

  return (int)n;                       // <0 is error code
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
