/* ---------------------------------------------------------------- */
//
// AppDelegate.m
// DCU
//
// Created by Christoph Birk on 2013-02-26
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#define DEBUG           1

#define FRAME_MAIN      "mainWindow"

/* ---------------------------------------------------------------- */

#import "AppDelegate.h"
#import "Notice.h"
#import "MTwister.h"
#import "Yutils.h"

/* ---------------------------------------------------------------- */

@implementation AppDelegate

@synthesize dcuController;

/* ---------------------------------------------------------------- */
#pragma mark "Application Delegate"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)applicationDidFinishLaunching:(NSNotification *)note
{
  NSString  *telname,*dcuname;
  int       value;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif
  assert(dcuController);              // dcuController:init has run
    
  if (!YappNapDisabled()) {           // b1.0203
    [Notice openNotice:"Warning: AppNap enabled\rPlease restart the GUI"
            title:P_TITLE time:0 level:1];
  }
  
  assert(popup_telname);
  telname = YprefGetString(@KEY_TELNAME,@"Clay",use_shared);
  // NSLog(@"%@ (%p)\n",telname,telname);
  if (telname) [popup_telname selectItemWithTitle:telname];
  else         [popup_telname selectItemWithTitle:@"Clay"];
  assert(popup_telstat);
  value = YprefGetInt(KEY_TELSTAT,1,use_shared);
  [popup_telstat selectItemAtIndex:value];
  if ([telname compare:@"Baade"] == NSOrderedSame) { 
    [check_mcal setState:NSOffState];
    [check_mcal setEnabled:NO];        
  }
  
  assert(combo_dcuname);
#ifdef SIM_ONLY
#ifndef NDEBUG
  [combo_dcuname addItemWithObjectValue:@"andromeda"];
#endif
  [combo_dcuname addItemWithObjectValue:@"localhost"];
#else
#ifndef NDEBUG
  [combo_dcuname addItemWithObjectValue:@"192.91.178.187"];
#endif
#endif
  dcuname = YprefGetString(@KEY_DCUNAME,nil,use_shared);
  // NSLog(@"%@ (%p)\n",dcuname,dcuname);
  if (dcuname) [combo_dcuname setStringValue:dcuname];
  assert(popup_dcustat);
  value = YprefGetInt(KEY_DCUSTAT,1,use_shared);
  [popup_dcustat selectItemAtIndex:value];

  assert(check_engmode);
#ifdef NDEBUG
  value = YprefGetInt(KEY_ENGMODE,0,use_shared);
  [check_engmode setState:((value) ? NSOnState : NSOffState)];
#endif

  assert(check_ffs);
  value = YprefGetInt(KEY_FFS_DSBLD,0,use_shared);
  [check_ffs setState:((value) ? NSOffState : NSOnState)];
  
  assert(check_mcal);
  value = YprefGetInt(KEY_MCAL_DSBLD,0,use_shared);
  [check_mcal setState:((value) ? NSOffState : NSOnState)];

  assert(dcuWindow);
  [dcuWindow setFrameUsingName:@FRAME_MAIN];
#ifdef NDEBUG
  [dcuWindow setStyleMask:([dcuWindow styleMask] & ~NSClosableWindowMask)];
#endif

  if (telname && dcuname) {            // got setup from preferences
#ifdef NDEBUG
    [self action_ok:self];
#endif
  } else {                             // interactive setup
    [self action_telname:popup_telname];
  }
}

/* ---------------------------------------------------------------- */

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

  return YES;
}

/* ---------------------------------------------------------------- */

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

#ifdef NDEBUG
  if ([dcuController running]) {
    [Notice openNotice:"DCU moving\rcancelled 'Quit'" title:P_TITLE time:30];
    return NSTerminateCancel;
  }
  if ([dcuController deployed]) { // [close] disabled in Release b1.0202
    if (YalertQuestion(P_TITLE,"DCU deployed - Quit anyway?",1,"No","Yes")) {
      return NSTerminateCancel;
    }
  }
#endif
  
  return NSTerminateNow;
}

/* ---------------------------------------------------------------- */

- (void)applicationWillTerminate:(NSNotification *)note
{  
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif 
  
  [dcuController shutdown];
  [dcuWindow saveFrameUsingName:@FRAME_MAIN];
}

/* ---------------------------------------------------------------- */
#pragma mark "Actions"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (IBAction)action_prefs:(id)sender
{
  if ([dcuController running]) return;
  
  assert([NSThread isMainThread]); 
  [window makeKeyAndOrderFront:self];
}

/* ---------------------------------------------------------------- */

- (IBAction)action_file:(id)sender        // b1.0203
{
  char title[64];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif 

  Ystr2chr([sender title],title,sizeof(title));
  if (strstr(title,"Logfile")) {
    assert(main_logger);
    [[NSWorkspace sharedWorkspace] openFile:main_logger.logfile];
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_telname:(id)sender;     // fixed 'IB' defaults b1.0100
{
  assert(sender == popup_telname);
  
  if ([[sender titleOfSelectedItem] compare:@"Baade"] == NSOrderedSame) {
    [combo_dcuname setStringValue:@"baadedcu"]; // 200.28.147.39
    [check_mcal setState:NSOffState];
    [check_mcal setEnabled:NO];    
  } else 
  if ([[sender titleOfSelectedItem] compare:@"Clay"] == NSOrderedSame) {
    [combo_dcuname setStringValue:@"claydcu"];  // 200.28.147.40
    [check_mcal setEnabled:YES];
    [check_mcal setState:NSOnState];
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_ok:(id)sender
{
  YprefPutString(@KEY_TELNAME,[popup_telname titleOfSelectedItem],use_shared);
  YprefPutInt(KEY_TELSTAT,(int)[popup_telstat indexOfSelectedItem],use_shared);
  YprefPutString(@KEY_DCUNAME,[combo_dcuname stringValue],use_shared);
  YprefPutInt(KEY_DCUSTAT,(int)[popup_dcustat indexOfSelectedItem],use_shared);

  YprefPutInt(KEY_ENGMODE,([check_engmode state]) ? 1 : 0,use_shared);
  YprefPutInt(KEY_FFS_DSBLD,[check_ffs state] ? 0 : 1,use_shared);
  YprefPutInt(KEY_MCAL_DSBLD,[check_mcal state] ? 0 : 1,use_shared);
  
  assert(dcuController);
  BOOL r = [dcuController configure];
  if (r) {
    assert([NSThread isMainThread]); 
    [window orderOut:self];
    [dcuWindow makeKeyAndOrderFront:self];
  }
}

/* ---------------------------------------------------------------- */
/* Menu - Help */

- (IBAction)action_help:(id)sender
{
  NSURL *url=nil;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  url = [NSURL URLWithString:@"http://instrumentation.obs.carnegiescience.edu/Software/DCU"];
  if (url) [[NSWorkspace sharedWorkspace] openURL:url];
}

/* ---------------------------------------------------------------- */
#pragma mark "Utils"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (NSString*)errorString:(int)err      // String b1.0301
{
  NSString *string = [DCUcontroller errorString:err];

  if (!string) {                       // fall-back to excuse b1.0212
    string = [NSString stringWithFormat:@"%@ (%d)",[self makeExcuse],err];
  }
  assert(string);
  
  return string;
}

/* --- */

- (NSString*)makeExcuse                // b1.0212
{
  static NSArray* arrays=nil;
  
  @synchronized(self) {
    if (!arrays) {
      NSString *file = [[NSBundle mainBundle] pathForResource:@"excuses" ofType:@"xml"];
      arrays = [[NSArray arrayWithContentsOfFile:file] retain];
    }
  }
  assert(arrays);
  if (!arrays) return @"no excuse";
  assert([arrays count] == 4);
  if ([arrays count] != 4) return @"invalid excuse";

  NSMutableString *excuse = [NSMutableString stringWithCapacity:64];
  
  for (int j=0; j<4; j++) {
    NSArray *a = [arrays objectAtIndex:j];
    NSString *word = [a objectAtIndex:[MTwister intValue:(int)[a count]]];
    [excuse appendString:word];
    if ((j == 2) && ([MTwister doubleValue:1.0] > 0.2)) break;
    [excuse appendString:@" "];
  }
  return excuse;
}

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
