/* ---------------------------------------------------------------- */
//
// DCUcontroller.h
// DCU
//
// Created by Christoph Birk on 2013-02-27
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import <Cocoa/Cocoa.h>

#import "FFwidgets.h"
#import "FFmodel.h"
#import "Logger.h"
#import "plcio.h"

/* ---------------------------------------------------------------- */

#define N_LAMPS         8

#define DCU_MCAL        0
#define DCU_FFS         1

#define KEY_ENGMODE     "eng_mode"
#define KEY_FFS_DSBLD   "ffs_disabled" 
#define KEY_MCAL_DSBLD  "mcal_disabled"

BOOL   use_shared;
Logger *main_logger;                   // b0200 

/* ---------------------------------------------------------------- */

@interface DCUcontroller : NSObject {
  IBOutlet NSWindow *window;           // main window
  IBOutlet NSImageView *view_in;
           NSImage  *icon_in;
           NSString *s_title;
           char     p_title[32];
           NSString *telname;
           char     telhost[128];
           PLC_IO   *plc;
           u_long   counter;
           u_int    cookie;
           BOOL     cookie_continue;
  volatile BOOL     configured;
           BOOL     use_interlocks,eng_mode;
           time_t   next_dog,next_air;
           int      interval_dog;
           float    air_pressure,min_pressure;
           FFcoil   *ff_enable,*mc_enable;
           int      selector_addr;
           int      cookie_addr;
           int      error_state;
           int      error_addr;
           BOOL     ff_disabled,mc_disabled;
  // GUI - lamps
           FFlamp   *lamps[N_LAMPS];
  IBOutlet NSButton *but_lamp1,*but_lamp2,*but_lamp3;
  IBOutlet NSButton *but_lamp4,*but_lamp5,*but_lamp6;
  IBOutlet NSButton *but_lamp7,*but_lamp8;
           NSArray  *but_lamps;
           NSImage  *icon_on,*icon_off,*icon_err;
  // GUI - variable lamps
           FFvarLamp     *q_lamp;
  IBOutlet NSPopUpButton *popup_quartz;
  IBOutlet NSTextField   *edit_quartz,*label_quartz;
  // GUI - FFS
           FFscreen      *ff_screen;
  IBOutlet NSPopUpButton *drop_ffs,*popup_ffbrake; 
  IBOutlet NSButton      *check_ffins,*check_ffret,*check_ffbrake;
  IBOutlet FFprogress    *prog_screen;
  
  // GUI - Mcal
           FFscreen      *mc_screen;
  IBOutlet NSPopUpButton *drop_mcal,*popup_mcbrake; 
  IBOutlet NSButton      *check_mcins,*check_mcret,*check_mcbrake;
  IBOutlet FFprogress    *prog_mcal;
  // GUI - Other
  IBOutlet NSTextField   *edit_air;
           FFsensor      *air_sensor;
}

@property (readonly,nonatomic) NSPopUpButton *drop_ffs,*drop_mcal;
@property (readonly)           int error_state;

/* ---------------------------------------------------------------- */

+ (NSString*)errorString:(int)err;

- (BOOL)configure;
-  (int)getFullStatus;
- (void)shutdown;
- (BOOL)running;
- (BOOL)deployed;
- (NSString*)ffsState;
- (NSString*)ffsLamps;

- (IBAction)action_lamp:(id)sender;
- (IBAction)action_ffs:(id)sender;
- (IBAction)action_mcal:(id)sender;

- (void)terminate:(const char*)text;
- (void)updateState:(BOOL)snd_lamps stage:(BOOL)snd_stage;
- (void)updateLamp:(int)lamp;
- (void)updateVar;
- (void)updateTCS:(BOOL)snd_lamps stage:(BOOL)snd_stage;
- (void)updateFFS;
- (void)updateMcal;

- (void)run_dog;

- (BOOL)checkAir:(BOOL)interactive;
-  (int)checkPLC:(BOOL)interactive;

/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
