/* ---------------------------------------------------------------- */
//
// FFwidgets.m
// DCU
//
// Created by Christoph Birk on 2013-03-15
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#import "FFwidgets.h"

/* ---------------------------------------------------------------- */

#define DEBUG           0              // debug level

/* ---------------------------------------------------------------- */

@implementation FFprogress

@synthesize counter;

- (id)initWithCoder:(NSCoder*)coder
{
#if (DEBUG > 1)
  fprintf(stderr,"%s(%p)\n",PREFUN,self);
#endif

  return [super initWithCoder:coder];  
}

// An awakeFromNib message is sent to each object loaded from the 
// archive, but only if it can respond to the message, and only 
// after all the objects in the archive have been loaded and initialized.
// When an object receives an awakeFromNib message, it is guaranteed 
// to have all its outlet instance variables set.

- (void)awakeFromNib
{
#if (DEBUG > 0)
  fprintf(stderr,"%s:%p\n",PREFUN,self);
#endif

  if ([[super class] instancesRespondToSelector:@selector(awakeFromNib)]) {
    [super awakeFromNib];
  }
  
  lock = [[NSLock alloc] init];
  counter = 0;
}

- (void)turnOn
{
  [lock lock];
  counter += 1;
  assert(counter > 0);
  if (counter == 1) [self startAnimation:self];
  [lock unlock];
}

- (void)turnOff
{
  [lock lock];
  assert(counter > 0);
  counter -= 1;
  if (counter == 0) [self stopAnimation:self];
  [lock unlock];
}

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
