/* ---------------------------------------------------------------- */
//
// AppDelegate.m
// DCU
//
// Created by Christoph Birk on 2013-02-26
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
/* ---------------------------------------------------------------- */

#define DEBUG           1

#define RUN_SERVER

#define FRAME_MAIN      "mainWindow"

/* ---------------------------------------------------------------- */

#import "AppDelegate.h"                // main.h
#import "Notice.h"
#import "MTwister.h"
#import "Yutils.h"

/* ---------------------------------------------------------------- */

@implementation AppDelegate
{
#ifdef RUN_SERVER
  TCPIP_Server *tcpipServer;           // NEW b0310
#endif
}

@synthesize dcuController;

static char *p_version,*p_build;       // Class variables 

/* ---------------------------------------------------------------- */
#pragma mark "Application Delegate"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (void)applicationDidFinishLaunching:(NSNotification *)note
{
  NSString  *telname,*dcuname;
  int       value;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif
  assert(dcuController);              // dcuController:init has run
    
  if (!YappNapDisabled()) {           // b0203
    [Notice openNotice:"Warning: AppNap enabled\rPlease restart the GUI"
            title:P_TITLE time:0 level:1];
  }
  YbundleInfo(NULL,&p_build,&p_version,NULL,NULL,NULL); // NEW b0310

  assert(popup_telname);
  telname = YprefGetString(@KEY_TELNAME,@"Clay",use_shared);
  // NSLog(@"%@ (%p)\n",telname,telname);
  if (telname) [popup_telname selectItemWithTitle:telname];
  else         [popup_telname selectItemWithTitle:@"Clay"];
  assert(popup_telstat);
  value = YprefGetInt(KEY_TELSTAT,1,use_shared);
  [popup_telstat selectItemAtIndex:value];
  if ([telname compare:@"Baade"] == NSOrderedSame) { 
    [check_mcal setState:NSControlStateValueOff];
    [check_mcal setEnabled:NO];        
  }
  
  assert(combo_dcuname);
#ifdef SIM_ONLY
#ifndef NDEBUG
  [combo_dcuname addItemWithObjectValue:@"andromeda"];
#endif
  [combo_dcuname addItemWithObjectValue:@"localhost"];
#else
#ifndef NDEBUG
  [combo_dcuname addItemWithObjectValue:@"192.91.178.187"];
#endif
#endif
  dcuname = YprefGetString(@KEY_DCUNAME,nil,use_shared);
  // NSLog(@"%@ (%p)\n",dcuname,dcuname);
  if (dcuname) [combo_dcuname setStringValue:dcuname];
  assert(popup_dcustat);
  value = YprefGetInt(KEY_DCUSTAT,1,use_shared);
  [popup_dcustat selectItemAtIndex:value];

  assert(check_engmode);
#ifdef NDEBUG
  value = YprefGetInt(KEY_ENGMODE,0,use_shared);
  [check_engmode setState:((value) ? NSControlStateValueOn : NSControlStateValueOff)];
#endif

  assert(check_ffs);
  value = YprefGetInt(KEY_FFS_DSBLD,0,use_shared);
  [check_ffs setState:((value) ? NSControlStateValueOff : NSControlStateValueOn)];
  
  assert(check_mcal);
  value = YprefGetInt(KEY_MCAL_DSBLD,0,use_shared);
  [check_mcal setState:((value) ? NSControlStateValueOff : NSControlStateValueOn)];

  assert(dcuWindow);
  [dcuWindow setFrameUsingName:@FRAME_MAIN];
#ifdef NDEBUG
  [dcuWindow setStyleMask:([dcuWindow styleMask] & ~NSWindowStyleMaskClosable)];
#endif

  if (telname && dcuname) {            // got setup from preferences
#ifdef NDEBUG
    [self action_startup:self];
#endif
  } else {                             // interactive setup
    [self action_telname:popup_telname];
  }
}

/* ---------------------------------------------------------------- */

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

  return YES;
}

/* ---------------------------------------------------------------- */

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)appl
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,appl);
#endif

#ifdef NDEBUG
  if ([dcuController running]) {
    [Notice openNotice:"DCU moving\rcancelled 'Quit'" title:P_TITLE time:30];
    return NSTerminateCancel;
  }
  if ([dcuController deployed]) { // [close] disabled in Release b0202
    if (YalertQuestion(P_TITLE,"DCU deployed - Quit anyway?",1,"No","Yes")) {
      return NSTerminateCancel;
    }
  }
#endif
  
  return NSTerminateNow;
}

/* ---------------------------------------------------------------- */

- (void)applicationWillTerminate:(NSNotification *)note
{  
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,note);
#endif 

#ifdef RUN_SERVER
  [tcpipServer stop];
#endif

  [dcuController shutdown];
  [dcuWindow saveFrameUsingName:@FRAME_MAIN];
}

/* ---------------------------------------------------------------- */
#pragma mark "Actions"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (IBAction)action_prefs:(id)sender
{
  if ([dcuController running]) return;
  
  assert([NSThread isMainThread]); 
  [window makeKeyAndOrderFront:self];
}

/* ---------------------------------------------------------------- */

- (IBAction)action_file:(id)sender        // b0203
{
  char title[64];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif 

  Ystr2chr([sender title],title,sizeof(title));
  if (strstr(title,"Logfile")) {
    assert(main_logger);
#if (MAC_OS_X_VERSION_MIN_REQUIRED > MAC_OS_X_VERSION_10_13)
    [[NSWorkspace sharedWorkspace] openURL:[NSURL fileURLWithPath:main_logger.logfile]]; // b0309
#else
    [[NSWorkspace sharedWorkspace] openFile:main_logger.logfile];
#endif
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_telname:(id)sender;     // fixed 'IB' defaults b0100
{
  assert(sender == popup_telname);
  
  if ([[sender titleOfSelectedItem] compare:@"Baade"] == NSOrderedSame) {
    [combo_dcuname setStringValue:@"baadedcu"]; // 200.28.147.39
    [check_mcal setState:NSControlStateValueOff];
    [check_mcal setEnabled:NO];    
  } else 
  if ([[sender titleOfSelectedItem] compare:@"Clay"] == NSOrderedSame) {
    [combo_dcuname setStringValue:@"claydcu"];  // 200.28.147.40
    [check_mcal setEnabled:YES];
    [check_mcal setState:NSControlStateValueOn]; // Mojave b0309
  } else {
    assert(0);
  }
}

/* ---------------------------------------------------------------- */

- (IBAction)action_startup:(id)sender         
{

  YprefPutString(@KEY_TELNAME,[popup_telname titleOfSelectedItem],use_shared);
  YprefPutInt(KEY_TELSTAT,(int)[popup_telstat indexOfSelectedItem],use_shared);
  YprefPutString(@KEY_DCUNAME,[combo_dcuname stringValue],use_shared);
  YprefPutInt(KEY_DCUSTAT,(int)[popup_dcustat indexOfSelectedItem],use_shared);

  YprefPutInt(KEY_ENGMODE,([check_engmode state]==NSControlStateValueOn) ? 1 : 0,use_shared);
  YprefPutInt(KEY_FFS_DSBLD,([check_ffs state]==NSControlStateValueOn) ? 0 : 1,use_shared);
  YprefPutInt(KEY_MCAL_DSBLD,([check_mcal state]==NSControlStateValueOn) ? 0 : 1,use_shared);
  
  assert(dcuController);
  BOOL r = [dcuController configure];
  if (!r) return;                   // 'terminate' was called in 'configure'
  
  assert([NSThread isMainThread]);
  [window orderOut:self];
  [dcuWindow makeKeyAndOrderFront:self];

#ifdef RUN_SERVER
  int port = 50001+100*PROJECT_ID;  // 51701
  tcpipServer = [[TCPIP_Server alloc] initWithPort:port delegate:self];
#ifdef NDEBUG
  tcpipServer.throttle = 0.5f;      // max. 2 connections per second
#endif
  dispatch_async(GLOBAL_QUEUE,^(void) { char buf[128];
    int err = [tcpipServer start];
    if (err) {
      sprintf(buf,"%s: TCP/IP server failed, err=%d",P_TITLE,err);
      [main_logger append:buf];
      YnotifyNotice(buf,[[self errorString:err] UTF8String],0,2);
      // dispatch_sync(MAIN_QUEUE,^(void) { [NSApp terminate:self]; }); // todo
    } else {
      sprintf(buf,"listening to port %d\n",port);
      [main_logger append:buf];
    }
  });
#endif
}

/* ---------------------------------------------------------------- */
/* Menu - Help */

- (IBAction)action_help:(id)sender
{
  NSURL *url=nil;
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,sender);
#endif

  url = [NSURL URLWithString:@"http://instrumentation.obs.carnegiescience.edu/Software/DCU"];
  if (url) [[NSWorkspace sharedWorkspace] openURL:url];
}

/* ---------------------------------------------------------------- */
#pragma mark "Utils"
/* ---------------------------------------------------------------- */
//34567890123456789012345678901234567890

- (NSString*)errorString:(int)err      // String b0301
{
  NSString *string = [DCUcontroller errorString:err];

  if (!string) {                       // fall-back to excuse b0212
    string = [NSString stringWithFormat:@"%@ (%d)",[self makeExcuse],err];
  }
  assert(string);
  
  return string;
}

/* --- */

- (NSString*)makeExcuse                // b0212
{
  static NSArray* arrays=nil;
  
  @synchronized(self) {
    if (!arrays) {
      NSString *file = [[NSBundle mainBundle] pathForResource:@"excuses" ofType:@"xml"];
      arrays = [[NSArray arrayWithContentsOfFile:file] retain];
    }
  }
  assert(arrays);
  if (!arrays) return @"no excuse";
  assert([arrays count] == 4);
  if ([arrays count] != 4) return @"invalid excuse";

  NSMutableString *excuse = [NSMutableString stringWithCapacity:64];
  
  for (int j=0; j<4; j++) {
    NSArray *a = [arrays objectAtIndex:j];
    NSString *word = [a objectAtIndex:[MTwister intValue:(int)[a count]]];
    [excuse appendString:word];
    if ((j == 2) && ([MTwister doubleValue:1.0] > 0.2)) break;
    [excuse appendString:@" "];
  }
  return excuse;
}

/* ---------------------------------------------------------------- */
#pragma mark "TCPIP_Server delegate"
/* ---------------------------------------------------------------- */

#ifdef RUN_SERVER
- (char*)tcpip_handler:(const char*)command from:(NSString*)ipNumber socket:(int)msgsock
{
  char cmd[128]="",par1[128]="",par2[128]="";
  NSString *string = @(command);
  NSString *line = [string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
#if (DEBUG > 0)
  fprintf(stderr,"%s(%s,%s,%d)\n",PREFUN,[line UTF8String],[ipNumber UTF8String],msgsock);
#endif

  char *answer = (char*)malloc(512*sizeof(char));
  *answer = '\0';
  if (strlen(command) >= sizeof(cmd)) return answer;

  string = [NSString stringWithFormat:@"TCP: %@-> %@",ipNumber,line];
  [main_logger append:[string UTF8String]];
  sscanf([line UTF8String],"%s %s %s",cmd,par1,par2);
  
  if (!strcmp(cmd,"version")) {        // version check -----------
    sprintf(answer,"%s (%s)",p_version,p_build);
  } else
  if (!strcmp(cmd,"ffsState")) {       // FF-Screen
    sprintf(answer,"%s",[[dcuController ffsState] UTF8String]);
  } else
  if (!strcmp(cmd,"ffsLamps")) {       // FF-Lamps
    sprintf(answer,"%s",[[dcuController ffsLamps] UTF8String]);
  } else {                             // unknown command
    sprintf(answer,"unknown command '%s'",cmd);
    Ysleep(1000);                      // throttle
  }
  assert(answer);

  string = [NSString stringWithFormat:@"TCP: %@<- %s",ipNumber,answer];
  [main_logger append:[string UTF8String]];
  strcat(answer,"\n");                 // terminating <LF>
  
  return answer;
}
#endif


/* ---------------------------------------------------------------- */

@end

/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
