/* ---------------------------------------------------------------- */
//
// scripting.m
// DCU
//
// Created by Christoph Birk on 2013-06-21
// Copyright 2013 Carnegie Observatories. All rights reserved.
//
// Note: defaults write NSGlobalDomain NSScriptingDebugLogLevel {0,1}
// Warning: ensure nothing is blocking -- commands run on the mainThread
//
/* ---------------------------------------------------------------- */

#define DEBUG      1                   // debug level

/* ---------------------------------------------------------------- */

#import  "scripting.h"

#import  "AppDelegate.h"
#import  "DCUcontroller.h"

#include <sys/time.h>

/* ---------------------------------------------------------------- */

static int flagFromString(NSString* string)

{
  if (!string) return 0;
  
  if ([string caseInsensitiveCompare:@"ffs"]  == NSOrderedSame) return DCU_FFS;
  if ([string caseInsensitiveCompare:@"mcal"] == NSOrderedSame) return DCU_MCAL;
  if ([string isEqualToString:@"FFS "]) return DCU_FFS;
  
  return -1;
}

/* ---------------------------------------------------------------- */

static double walltime(void)
{
  struct timeval tp;
  
  gettimeofday(&tp,NULL);
  
  return((double)(tp.tv_sec)+(double)(tp.tv_usec)/1.0e6);
}

/* ---------------------------------------------------------------- */

@implementation DCUcommand_Deploy

- (id)performDefaultImplementation
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,self);
#endif
  DCUcontroller *dcuCon=((AppDelegate*)[NSApp delegate]).dcuController;

  NSString *dcu = [self directParameter]; // NSLog(@"%@",dcu);
  if (!dcu) return nil;
  
  switch (flagFromString(dcu)) {
  case DCU_FFS:                        // deploy FFS
    [dcuCon.drop_ffs selectItemWithTitle:@"Deploy"];
    [dcuCon action_ffs:self];
    break;
  case DCU_MCAL:                       // deploy MCAL
    [dcuCon.drop_mcal selectItemWithTitle:@"Deploy"];
    [dcuCon action_mcal:self];
    break;
  }  
  // [NSThread sleepForTimeInterval:0.350];  // KVO on mainThread v0.905
  
  return nil;
}

@end

/* ---------------------------------------------------------------- */

@implementation DCUcommand_Retract

- (id)performDefaultImplementation
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,self);
#endif
  DCUcontroller *dcuCon=((AppDelegate*)[NSApp delegate]).dcuController;

  NSString *dcu = [self directParameter]; // NSLog(@"%@",dcu);
  if (!dcu) return nil;
  
  switch (flagFromString(dcu)) {
  case DCU_FFS:                        // retract FFS
    [dcuCon.drop_ffs selectItemWithTitle:@"Retract"]; // Note: 'IB' dependency
    [dcuCon action_ffs:self];
    break;
  case DCU_MCAL:                       // retract MCAL
    [dcuCon.drop_mcal selectItemWithTitle:@"Retract"];
    [dcuCon action_mcal:self];
    break;
  }
  // [NSThread sleepForTimeInterval:0.350];  // KVO on mainThread v0.905
  
  return nil;
}

@end

/* ---------------------------------------------------------------- */

@implementation M2FSCommand_Sync

- (id)performDefaultImplementation
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,self);
#endif
  DCUcontroller *dcuCon=((AppDelegate*)[NSApp delegate]).dcuController;

  [NSThread sleepForTimeInterval:0.050];

  return [NSNumber numberWithBool:[dcuCon running]];
}

@end

/* ---------------------------------------------------------------- */

@implementation M2FSCommand_Error

- (id)performDefaultImplementation
{
#if (DEBUG > 0)
  fprintf(stderr,"%s(%p)\n",PREFUN,self);
#endif
  DCUcontroller *dcuCon=((AppDelegate*)[NSApp delegate]).dcuController;

  return [NSNumber numberWithInt:dcuCon.error_state];
}

@end


/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
/* ---------------------------------------------------------------- */
